#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>

#include <iostream>

#include <gui/window.h>
#include <gui/layoutview.h>
#include <gui/button.h>
#include <gui/menu.h>
#include <gui/listview.h>
#include <gui/tabview.h>
#include <gui/imagebutton.h>
#include <gui/frameview.h>
#include <gui/stringview.h>
#include <gui/separator.h>
#include <gui/textview.h>
#include <gui/requesters.h>
#include <gui/image.h>
#include <gui/point.h>
#include <gui/guidefines.h>
#include <util/application.h>
#include <util/message.h>
#include <util/settings.h>
#include <util/string.h>
#include <util/invoker.h>
#include <util/datetime.h>
#include <util/exceptions.h>
#include <storage/file.h>
#include <storage/path.h>
#include <storage/fsnode.h>
#include <storage/directory.h>
#include <storage/filereference.h>

class CommThread;

class MainWindow:public os::Window
{
public:
	MainWindow();
	virtual void HandleMessage( os::Message *pcMessage );
	virtual bool OkToQuit();

private:
	void SetBookmarkPath();
	void LoadBookmarkList( os::Directory *pcDirectory );
	void AddBookmarkToList();
	void RemoveBookmarkFromList();
	void SelectBookmarkFromList();
	os::BitmapImage *LoadImageFromResource( os::String zResource );
	void AddStringToTextView( const char* pzName ) const;
	void AboutRequested();

	os::VLayoutNode* m_pcRoot;
	os::VLayoutNode* m_pcVRoot;
	os::Menu* m_pcMenu;
	os::HLayoutNode* m_pcHLayoutNode;
	os::VLayoutNode* m_pcSideNode;
	os::TabView* m_pcSideTab;
	os::LayoutView* m_pcBookmarksTab;
	os::VLayoutNode* m_pcBookmarksLayout;
	os::LayoutView* m_pcUserListTab;
	os::VLayoutNode* m_pcUserListLayout;
	os::VLayoutNode* m_pcBookmarks;
	os::FrameView* m_pcBookmarkFrame;
	os::VLayoutNode* m_pcBookmarkFrameLayout;
	os::TextView* m_pcTextBookmarkName;
	os::VLayoutSpacer* m_pcBookmarkSpacer1;
	os::Button* m_pcButtonBookmarkName;
	os::VLayoutSpacer* m_pcBookmarkSpacer2;
	os::ListView* m_pcBookmarkList;
	os::VLayoutSpacer* m_pcBookmarkSpacer3;
	os::HLayoutNode* m_pcBookmarkButtonsNode;
	os::ImageButton* m_pcButtonRemoveBookmark;
	os::HLayoutSpacer* m_pcBookmarkSpacer4;
	os::Button* m_pcButtonBConnect;
	os::VLayoutNode* m_pcUserList;
	os::VLayoutNode* m_pcMainNode;
	os::HLayoutNode* m_pcConnectNode;
	os::TabView* m_pcConnectTab;
	os::LayoutView* m_pcUserSettingsTab;
	os::VLayoutNode* m_pcUserSettingsLayout;
	os::LayoutView* m_pcServerSettingsTab;
	os::VLayoutNode* m_pcServerSettingsLayout;
	os::HLayoutNode* m_pcUserSettings;
	os::VLayoutNode* m_pcUserNameNode;
	os::StringView* m_pcStringUserName;
	os::VLayoutSpacer* m_pcUserNameSpacer;
	os::TextView* m_pcTextUserName;
	os::VLayoutNode* m_pcUserPasswordNode;
	os::StringView* m_pcStringUserPassword;
	os::VLayoutSpacer* m_pcUserPasswordSpacer;
	os::TextView* m_pcTextUserPassword;
	os::VLayoutNode* m_pcUserRealNameNode;
	os::StringView* m_pcStringUserRealName;
	os::VLayoutSpacer* m_pcUserRealNameSpacer;
	os::TextView* m_pcTextUserRealName;
	os::HLayoutNode* m_pcServerSettings;
	os::VLayoutNode* m_pcServerNameNode;
	os::StringView* m_pcStringServerName;
	os::VLayoutSpacer* m_pcServerNameSpacer;
	os::TextView* m_pcTextServerName;
	os::VLayoutNode* m_pcServerPortNode;
	os::StringView* m_pcStringServerPort;
	os::VLayoutSpacer* m_pcServerPortSpacer;
	os::TextView* m_pcTextServerPort;
	os::VLayoutNode* m_pcServerChannelNode;
	os::StringView* m_pcStringServerChannel;
	os::VLayoutSpacer* m_pcServerChannelSpacer;
	os::TextView* m_pcTextServerChannel;
	os::VLayoutNode* m_pcConnectButtonsNode;
	os::VLayoutSpacer* m_pcConnectSpacer1;
	os::Button* m_pcButtonConnect;
	os::VLayoutSpacer* m_pcConnectSpacer2;
	os::Button* m_pcButtonDisconnect;
	os::TextView* m_pcTextView;
	os::HLayoutNode* m_pcMessageNode;
	os::TextView* m_pcMessageText;
	os::VLayoutNode* m_pcMessageButtonsNode;
	os::Button* m_pcButtonMessageSend;
	os::VLayoutSpacer* m_pcMessageSpacer;
	os::Button* m_pcButtonTextClear;

	os::Directory *m_pcDirectory;
	os::String stdName;
	os::String stdPassword;
	os::String stdRealName;
	os::String stdServerName;
	os::String stdServerPort;
	os::String stdServerChannel;

	CommThread *m_CommThread;
};

#endif

